"use strict";

/* exported logger */

var logger = require('../../logger');

var UserPreferences = require('./UserPreferences');
var ObjectWithListeners = require('../../ObjectWithListeners');

function User(javaObject) {
  // call super constructor
  ObjectWithListeners.call(this);

  this.registerListenerType("onreload");

  this.setLogin(javaObject.login);
  this.setName(javaObject.name);
  this.setSurname(javaObject.surname);
  this.setEmail(javaObject.email);
  this.setRemoved(javaObject.removed);
  this.setPrivileges(javaObject.privileges);
  this.setPreferences(javaObject.preferences);
  this.setMinColor(javaObject.minColor);
  this.setMaxColor(javaObject.maxColor);
  this.setNeutralColor(javaObject.neutralColor);
  this.setSimpleColor(javaObject.simpleColor);
}

// this class inherits from ObjectWithListeners class where generic methods for
// listeners are set
User.prototype = Object.create(ObjectWithListeners.prototype);
User.prototype.constructor = User;

User.prototype.setLogin = function (login) {
  this._login = login;
};

User.prototype.getLogin = function () {
  return this._login;
};

User.prototype.setName = function (name) {
  this._name = name;
};

User.prototype.getName = function () {
  return this._name;
};

User.prototype.setSurname = function (surname) {
  this._surname = surname;
};

User.prototype.getSurname = function () {
  return this._surname;
};

User.prototype.setEmail = function (email) {
  this._email = email;
};

User.prototype.getEmail = function () {
  return this._email;
};

User.prototype.setRemoved = function (removed) {
  this._removed = removed;
};

User.prototype.getRemoved = function () {
  return this._removed;
};

User.prototype.setMinColor = function (minColor) {
  this._minColor = minColor;
};

User.prototype.getMinColor = function () {
  return this._minColor;
};

User.prototype.setSimpleColor = function (simpleColor) {
  this._simpleColor = simpleColor;
};

User.prototype.getSimpleColor = function () {
  return this._simpleColor;
};

User.prototype.setNeutralColor = function (neutralColor) {
  this._neutralColor = neutralColor;
};

User.prototype.getNeutralColor = function () {
  return this._neutralColor;
};

User.prototype.setMaxColor = function (maxColor) {
  this._maxColor = maxColor;
};

User.prototype.getMaxColor = function () {
  return this._maxColor;
};

User.prototype.setPassword = function (password) {
  this._password = password;
};

User.prototype.getPassword = function () {
  return this._password;
};

User.prototype.setPrivileges = function (privileges) {
  this._privileges = privileges;
  if (this._privileges === undefined) {
    this._privileges = [];
  }
};

User.prototype.getPrivileges = function () {
  return this._privileges;
};

User.prototype.setPreferences = function (preferences) {
  if (!(preferences instanceof UserPreferences)) {
    preferences = new UserPreferences(preferences);
  }
  this._preferences = preferences;
};

User.prototype.getPreferences = function () {
  return this._preferences;
};

User.prototype.hasPrivilege = function (type, objectId) {
  return this.getPrivilegeValue(type, objectId) > 0;
};

User.prototype.setPrivilege = function (params) {
  var objectId = params.objectId;
  var type = params.type;
  var value = params.value;
  if (value === undefined) {
    value = 1;
  } else if (value === true) {
    value = 1;
  } else if (value === false) {
    value = 0;
  }
  for (var i = 0; i < this._privileges.length; i++) {
    var privilege = this._privileges[i];
    if (privilege.type === type.getName()) {
      if (objectId === privilege.objectId) {
        privilege.value = value;
        return;
      }
    }
  }
  this._privileges.push({
    type: type.getName(),
    value: value,
    objectId: objectId
  });
};

User.prototype.getPrivilegeValue = function (type, objectId) {
  for (var i = 0; i < this._privileges.length; i++) {
    var privilege = this._privileges[i];
    if (privilege.type === type.getName()) {
      if (objectId === privilege.objectId) {
        return privilege.value;
      }
    }
  }
  return 0;
};

User.prototype.privilegesToExport = function (configuration) {
  var self = this;
  var result = {};
  var validTypes = {};
  var i;
  for (i = 0; i < configuration.getPrivilegeTypes().length; i++) {
    validTypes[configuration.getPrivilegeTypes()[i].getName()] = true;
  }
  for (i = 0; i < self.getPrivileges().length; i++) {
    var privilege = self.getPrivileges()[i];
    if (validTypes[privilege.type]) {
      if (result[privilege.type] === undefined) {
        result[privilege.type] = {};
      }
      if (privilege.objectId !== undefined) {
        result[privilege.type][privilege.objectId] = privilege.value;
      } else {
        result[privilege.type] = privilege.value;
      }
    }
  }
  return result;
};

User.prototype.update = function (user) {
  var self = this;
  self.setLogin(user.getLogin());
  self.setName(user.getName());
  self.setSurname(user.getSurname());
  self.setEmail(user.getEmail());
  self.setRemoved(user.getRemoved());
  self.setPrivileges(user.getPrivileges());
  self.getPreferences().update(user.getPreferences());
  self.setMinColor(user.getMinColor());
  self.setMaxColor(user.getMaxColor());
  self.setSimpleColor(user.getSimpleColor());

  return self.callListeners("onreload");
};

module.exports = User;
